COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1992 -- All Rights Reserved

PROJECT:	PC GEOS
MODULE:		
FILE:		vobj.asm

AUTHOR:		John Wedgwood, Jun 27, 1992

ROUTINES:
	Name			Description
	----			-----------
	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	John	 6/27/92	Initial revision

DESCRIPTION:
	Implementation of the VObj class

	$Id: vobj.asm,v 1.1.12.1 93/05/11 08:30:06 steve Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
idata	segment
	VObjClass		; have to put the class definition somewhere...
idata	ends

CommonCode	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjShuffleColor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Shuffle the color to be something else.

CALLED BY:	via MSG_VOBJ_SHUFFLE_COLOR
PASS:		*ds:si	= Instance
		ds:di	= Instance
RETURN:		nothing
DESTROYED:	everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjShuffleColor	method	VObjClass, MSG_VOBJ_SHUFFLE_COLOR
	mov	cl, ds:[di].VOI_color		; cl <- old color
	inc	cl				; Move to next one

	cmp	cl, Color			; Make sure next is legal
	jb	colorOK				; Branch if it is
	clr	cl				; Otherwise wrap to zero

colorOK:
	mov	ds:[di].VOI_color, cl		; Save new color
	
	call	ForceVObjRedraw
	ret
VObjShuffleColor	endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjSetColor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the color of an objct.

CALLED BY:	via MSG_VOBJ_SET_COLOR
PASS:		*ds:si	= Instance
		ds:di	= Instance
		cl	= New color
RETURN:		nothing
DESTROYED:	everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjSetColor	method	VObjClass, MSG_VOBJ_SET_COLOR
	mov	ds:[di].VOI_color, cl		; Save new color
	call	ForceVObjRedraw
	ret
VObjSetColor	endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjSetType
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the type of the object.

CALLED BY:	via MSG_VOBJ_SET_TYPE
PASS:		*ds:si	= Instance
		ds:di	= Instance
		cx	= VObjType
RETURN:		nothing
DESTROYED:	everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjSetType	method	VObjClass, MSG_VOBJ_SET_TYPE
	mov	ds:[di].VOI_type, cx		; Save new type
	call	ForceVObjRedraw
	ret
VObjSetType	endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ForceVObjRedraw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Force an object to redraw.

CALLED BY:	
PASS:		*ds:si	= Instance
RETURN:		nothing
DESTROYED:	everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ForceVObjRedraw	proc	near
	mov	ax, MSG_VIS_INVALIDATE
	call	ObjCallInstanceNoLock
	ret
ForceVObjRedraw	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjDraw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Draw an object

CALLED BY:	via MSG_VIS_DRAW
PASS:		*ds:si	= Instance
		ds:di	= Instance
		bp	= GState to use
RETURN:		nothing
DESTROYED:	everything

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjDraw	method	VObjClass, MSG_VIS_DRAW
	;
	; Get the gstate to use.
	;
	mov	si, di				; ds:si <- our instance data
	mov	di, bp				; di <- gstate to use

	;
	; Set the appropriate color
	;
	clr	ah
	mov	al, ds:[si].VOI_color
	call	GrSetAreaColor

	;
	; Load up the coordinates for drawing.
	;
	mov	ax, ds:[si].VI_bounds.R_left	; ax...dx <- bounds of object
	mov	bx, ds:[si].VI_bounds.R_top
	mov	cx, ds:[si].VI_bounds.R_right
	mov	dx, ds:[si].VI_bounds.R_bottom
	
	;
	; Call the handler appropriate for the tool-type
	;
	mov	bp, ds:[si].VOI_type
	call	cs:vobjDrawHandlerTable[bp]	; Call the handler

	ret

;
; List of handlers for each drawing tool.
;
vobjDrawHandlerTable	word	\
	offset	cs:DrawRectangleObject,		; VOT_RECTANGLE
	offset	cs:DrawEllipseObject		; VOT_ELLIPSE

VObjDraw	endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DrawRectangleObject
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Draw a rectangle object.

CALLED BY:	VObjDraw
PASS:		ds:si	= Instance
		ax...dx	= Bounds to draw to
		di	= GState to draw with (color already set)
RETURN:		nothing
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DrawRectangleObject	proc	near
	call	GrFillRect
	ret
DrawRectangleObject	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		DrawEllipseObject
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Draw an ellipse object.

CALLED BY:	VObjDraw
PASS:		ds:si	= Instance
		ax...dx	= Bounds to draw to
		di	= GState to draw with (color already set)
RETURN:		nothing
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DrawEllipseObject	proc	near
	call	GrFillEllipse
	ret
DrawEllipseObject	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjInitialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the object.

CALLED BY:	via MSG_META_INITIALIZE
PASS:		*ds:si	= Instance
		ds:di	= Instance
		es	= Segment containing class
RETURN:		nothing
DESTROYED:	di

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjInitialize	method	VObjClass, MSG_META_INITIALIZE
	;
	; Call superclass to do initialization.
	;
	mov	di, offset es:VObjClass
	call	ObjCallSuperNoLock
	
	;
	; Clear some bits so our object will come up in the way that the
	; system expects...
	;
	mov	di, ds:[si]
	add	di, ds:[di].Vis_offset
	
	and	ds:[di].VI_attrs, not mask VA_REALIZED
	and	ds:[di].VI_optFlags, not mask VOF_IMAGE_INVALID
	ret
VObjInitialize	endm


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		VObjStartSelect
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle start-select

CALLED BY:	via MSG_META_START_SELECT
PASS:		*ds:si	= Instance
		ds:di	= Instance
		cx	= X position of event
		dx	= Y position of event
RETURN:		nothing
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	 6/27/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
VObjStartSelect	method	VObjClass, MSG_META_START_SELECT
	;
	; Shuffle the color when we're clicked on
	;
	mov	ax, MSG_VOBJ_SHUFFLE_COLOR
	call	ObjCallInstanceNoLock
	
	;
	; Tell the caller that we handled this event
	;
	mov	ax, mask MRF_PROCESSED
	ret
VObjStartSelect	endm

CommonCode	ends


