COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1993 -- All Rights Reserved

PROJECT:	
MODULE:		
FILE:		cidserEntry.asm

AUTHOR:		Adam de Boor, Sep 29, 1993

ROUTINES:
	Name			Description
	----			-----------

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	Adam	9/29/93		Initial revision


DESCRIPTION:
	The three entry points for this driver.
		

	$Id: cidserEntry.asm,v 1.1.1.2 93/10/28 20:31:08 adam Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

Resident	segment	resource


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerPStrategy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Strategy routine for supporting PCMCIA driver functions

CALLED BY:	GLOBAL
PASS:		di	= PCMCIAFunction
RETURN:		varies
DESTROYED:	
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/13/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
DefPFunction	macro	routine, constant
.assert ($-pfuncs) eq constant*2, <Routine for constant in the wrong slot>
.assert (type routine eq far), <routine should be far>
		fptr	routine
		endm

pfuncs		label	fptr.far
DefPFunction CIDSerEInit,			DR_INIT
DefPFunction CIDSerEExit,			DR_EXIT
	; These next two should unregister & register, should they ever be
	; needed...
DefPFunction CIDSerEDoNothing,			DR_SUSPEND
DefPFunction CIDSerEDoNothing,			DR_UNSUSPEND
DefPFunction CIDSerICheckSocket,		DR_PCMCIA_CHECK_SOCKET
DefPFunction CIDSerRObjectionResolved,		DR_PCMCIA_OBJECTION_RESOLVED
DefPFunction CIDSerRCloseSocket,		DR_PCMCIA_CLOSE_SOCKET
DefPFunction CIDSerEDoNothing,			DR_PCMCIA_DEVICE_ON
DefPFunction CIDSerEDoNothing,			DR_PCMCIA_DEVICE_OFF
	.assert ($-pfuncs) eq PCMCIAFunction*2

CIDSerPStrategy	proc	far
		uses	ds, es
		.enter
		segmov	ds, dgroup, ax
		mov	ss:[TPD_dataBX], bx
		mov	es, ax
		shl	di
		movdw	bxax, cs:[pfuncs][di]
		call	ProcCallFixedOrMovable
		.leave
		ret
CIDSerPStrategy	endp

CIDSerEDoNothing	proc	far
		clc
		ret
CIDSerEDoNothing	endp

Init	segment	resource

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerEInit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Make sure we can actually function.

CALLED BY:	DR_INIT
PASS:		ds, es 	= dgroup
RETURN:		carry set on error
DESTROYED:	ax, bx, cx, dx, ds, si
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	5/13/93		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerEInit 	proc	far
		.enter
	;
	; Fetch the strategy routine for the serial driver now, as fetching
	; it involves loading in the core block, which we can't do at interrupt
	; time.
	; 
		mov	bx, handle serial
		call	GeodeInfoDriver
		
		movdw	bxax, ds:[si].DIS_strategy

		segmov	ds, es		; ds <- dgroup

		movdw	ds:[serialStrat], bxax
	;
	; Register as a card-services client.
	; 
		mov	di, segment CIDSerCardServicesCallback
		mov	si, offset CIDSerCardServicesCallback
		mov	cx, size regArgList
		segmov	es, cs, ax
		mov	bx, offset regArgList
		CallCS	CSF_REGISTER_CLIENT
		jc	fail

		mov	ds:[csHandle], CS_HANDLE_REG
		clc
done:
		.leave
		ret
fail:
		stc
		jmp	done
CIDSerEInit 	endp

	; we manage an I/O device, but we register as a memory client to make
	; sure Card ID has a chance to configure the card before we get called
regArgList	CSRegisterClientArgs <
	mask CSRCAA_ARTIFICIAL_EXCLUSIVE or mask CSRCAA_ARTIFICIAL_SHARED or \
	    mask CSRCAA_MCD,
	mask CSEM_CARD_DETECT_CHANGE,
	< 0, segment dgroup, 0, 0>,
	0201h
>
Init	ends



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		CIDSerEExit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Clean up after ourselves

CALLED BY:	DR_EXIT
PASS:		ds	= dgroup (from strategy routine)
RETURN:		nothing
DESTROYED:	nothing
SIDE EFFECTS:	driver unregistered as FSD and as card services client

PSEUDO CODE/STRATEGY:
		XXX: should probably check for error from FSDUnregister and
		make sure we've deleted any drives...

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 4/93	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CIDSerEExit	proc	far
		uses	ax, dx
		.enter

		mov	dx, ds:[csHandle]
		CallCS	CSF_DEREGISTER_CLIENT
		
		clc
		.leave
		ret
CIDSerEExit	endp

Resident	ends
