COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1992 -- All Rights Reserved

PROJECT:	PC GEOS
MODULE:		Startup Sound
FILE:		manager.asm

AUTHOR:		Andrew Wilson

	
DESCRIPTION:
	
	This is the glue for all the modules		

	$Id: manager.asm,v 1.1.1.1 93/08/24 11:28:55 andrew Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

;------------------------------------------------------------------------------
;			    Include Files
;------------------------------------------------------------------------------

include	geos.def
include	heap.def
include	geode.def
include	resource.def
include	ec.def
include thread.def
include sem.def
include timer.def
include driver.def
include system.def
include disk.def
include Objects/processC.def

include variable.def

UseLib wav.def

StartSndProcessClass	class	ProcessClass
; Our process class - this thread plays a sound.
StartSndProcessClass	endc

idata	segment
	StartSndProcessClass	mask CLASSF_NEVER_SAVED
idata	ends

;------------------------------------------------------------------------------
;			    Code
;------------------------------------------------------------------------------

Resident	segment	resource



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		StartSoundStrategy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handles the various driver calls.

CALLED BY:	GLOBAL
PASS:		di - DriverFunction
RETURN:		nada
DESTROYED:	nada
 
PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	atw	7/20/93   	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
StartSoundStrategy	proc	far
	.enter

;	Every call except for TEST_DEVICE will succeed - we fail the
;	TEST_DEVICE so we'll be unloaded immediately by the FATFS driver.
;	We won't exit until our process thread exits, though.

	cmp	di, DRE_TEST_DEVICE
	stc
	je	exit
	clc
exit:
	.leave
	ret
StartSoundStrategy	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		StartSndAttach
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Tries to play a sound, then exits.

CALLED BY:	GLOBAL
PASS:		nada
RETURN:		nada
DESTROYED:	nada
 
PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	atw	7/21/93   	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
soundFileName	char	"STARTUP.WAV",0
soundDirName	char	"SOUNDS",0
StartSndAttach	method	StartSndProcessClass, MSG_META_ATTACH

	push	ax, cx, dx, bp

;
;	Change us to be a high-priority thread, so the sound gets played with
;	(hopefully) no interruptions.
;
	clr	bx
	mov	ax, PRIORITY_HIGH or mask TMF_ZERO_USAGE or mask TMF_BASE_PRIO
	call	ThreadModify

;
;	Try to play the "STARTUP.WAV" sound in the USERDATA/SOUNDS directory
;

	push	ds
	segmov	ds, cs
	call	FilePushDir
	mov	bx, SP_USER_DATA
	mov	dx, offset soundDirName
	call	FileSetCurrentPath
	jc	err
	
	mov	dx, offset soundFileName
	mov	al, FILE_ACCESS_R or FILE_DENY_NONE
	call	FileOpen

err:
	call	FilePopDir
	pop	ds
	jc	exit
	mov_tr	bx, ax			;BX <- sound file handle

	call	PlaySoundFromFile

	mov	al, FILE_NO_ERRORS
	call	FileClose
exit:
;
;	We're done - pass the MSG_META_ATTACH on to the superclass, then
;	nuke the thread/geode
;

	pop	ax, cx, dx, bp
	mov	di, offset StartSndProcessClass
	call	ObjCallSuperNoLock

	mov	ax, MSG_META_DETACH
	clr	cx, dx, bp
	mov	di, offset StartSndProcessClass
	GOTO	ObjCallSuperNoLock
StartSndAttach	endp

Resident	ends



