COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	Copyright (c) GeoWorks 1991 -- All Rights Reserved

PROJECT:	PC GEOS
MODULE:		
FILE:		pcget.asm

AUTHOR:		jimmy lefkowitz

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

DESCRIPTION:
	Code for retrieving a file via the serial port.

	The file transfer is performed using the following protocol:
	(See complete documentation in Tools section 10.8.3)

	sender: <Esc>XF 2 filename\0	     (name may contain wildcard chars)
	receiver: sync
	receiver: sync
	receiver: filename\0		     (destination filename)
	sender: sync
	receiver: file size
loop:
	receiver: BLOCK_START block_data BLOCK_END CRC
	sender: sync
	loop until all blocks sent
	receiver: checksum (0)
	sender: ack


	If a block is not successfully sent, it is tried again up to three 
	times, at which point we give up.
 
	all multi-byte values are send low byte to high byte
	checksum is a 4 byte quantity	
	packets are any length, up to 1K
	sync is ffh
	ack = 0 for ok, 1 for not ok


	$Id: pcget.asm,v 1.5 94/01/22 14:45:31 cassie Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

include com43.def

program	segment


		org	100h	; Must start at 100h

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		Main
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Entry point for pcget

CALLED BY:	MS-DOS

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Everything

PSEUDO CODE/STRATEGY:
	Calls appropriate initialization routines then goes into an infinite
	loop checking the the keyboard and the serial line, in that order, for
	characters that need to be transfered from the one to the other.

	Return from this function occurs when exit is called, at which point
	things are cleaned up and the appropriate trap back into MS-DOS is
	performed.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

Main		proc 	far	
		.enter
		call	MainCommon
getFileLoop:
		call	GetNextFileName
		jc	done
		call	GetFile
		jmp	getFileLoop
done:
	.leave
		call	ComExit

		mov	al, 0		; exit(0)
		mov	ah, DOS_EXIT
		int	21h
		.unreached
		ret
Main		endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ReadFileSize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	read the dword file size

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		dx:ax

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/11/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ReadFileSize	proc	far
	uses	bx
	.enter

	; GET THE SIZE OF THE INCOMING FILE
	call	ComReadWithWait
	jc	abort
	mov	dl, al
	call	ComReadWithWait
	jc	abort
	mov	dh, al			; dx = low word
	call	ComReadWithWait
	jc	abort
	mov	bl, al
	call	ComReadWithWait
	jc	abort
	mov	bh, al			; bx = high word
	mov	ax, bx
	xchg	ax, dx			; dx:ax = high word:low word
abort:
	.leave
	ret
	
ReadFileSize	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetFile
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get a file from com43

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	use the very simple file transfer protocol

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

fileflag	byte	'f',0
destname	byte	DOS_FILENAME_LENGTH+1	dup(?)
fileHandle	word	?
buf		byte	BUFFER_SIZE 	dup(?)
filesize	dd	?

GetFile	proc	near
tries	local	word
	.enter

	mov	tries, 0
	mov	cx, 1				; raw mode
	clr	ax				; don't flush anything
	call	ComSetRawMode

	; first send out an escape sequence telling com43 that a file
	; is being requested

	mov	al, 1bh		; esacpe code
	call	ComWrite
	mov	al, 'X'
	call	ComWrite
	mov	al, 'F'
	call	ComWrite
	mov	al, FETCH_RAW_COMMAND		; get a file
	call	ComWrite
	lea	bx, filename
	mov	ah, SCREEN_ATTR_NORMAL
nameLoop:
	mov	al, cs:[bx]
	push	ax
	call	ComWrite		
	pop	ax
	inc	bx
	tst	al
	jnz	nameLoop
firstSync::
	call	Sync
	LONG	jc	abort
	cmp	ax, NAK
	LONG	je	abort
	cmp	ax, NAK_QUIT
	LONG	je	abort
getCommand::

	; the second sync is after com43 has determined what type of
	; file it is dealing with, this takes a while...
	call	Sync
	LONG	jc	abort
	cmp	ax, NAK
	LONG	je	abort
	cmp	ax, NAK_QUIT
	LONG	je	abort
	call	ComReadWithWait
	LONG	jc	abort
	cmp	al, RECV_COMMAND
	je	createFile

	mov	al, NAK
	call	ComWrite
	jmp	Exit
	; now read in the filename and open the file	
createFile:
	mov	di, offset destname
	clr	cx
filenameLoop:
	call	ComReadWithWait
	LONG	jc	abort
	cmp	cx, size destname	; room for more chars in buffer?
	LONG	je	abort		; nope, we're outta here
	mov	cs:[di], al
	inc	di
	inc	cx
	tst	al
	jnz	filenameLoop

	mov	ah, DOS_FILE_CREATE	; create/truncate
	mov	cx, 0		; file attributes (normal)
	mov	dx, offset destname
	push	cs
	pop	ds
	int	21h		; ax <- file handle
	LONG	jc	fileCreateError
	mov	cs:[fileHandle], ax

	; sync back up by sending a sync
	mov	al, SYNC
	call	ComWrite

	; GET THE SIZE OF THE INCOMING FILE
	call	ReadFileSize		; dx:ax = file size
	LONG	jc	abort
	mov	cs:[filesize].low, ax
	mov	cs:[filesize].high, dx
	mov	cs:[checksum], 0

readBlock:
	mov	di, offset buf
	call	FileReadBlock	; cx = actual number of bytes to read
	jnc	writeData

	;
	; Flush the input buffer so there is no junk in it if
	; the sender tries to send it again.
	;
	mov	cx, 1			; (re)set raw mode
	mov	ax, 1			; flush input buffer only
	call	ComSetRawMode		; set the raw flag and flush buffer
	mov	al, NAK			; there was an error so send
	call	ComWrite		; a NAK and try again

	cmp	tries, MAX_TRIES	; of course, only try so many times
	jae	abort
	inc	tries
	jmp	readBlock

writeData:
	; WRITE THE STUFF OUT TO THE FILE
	mov	ah, DOS_FILE_WRITE
	mov	bx, cs:[fileHandle]
	mov	dx, offset buf
	int	21h				; FILE WRITE
	jnc	sendSync			; no file error so OK
	mov	err, 1				; else send a NAK_QUIT
	mov	al, NAK_QUIT			; and exit, resending a packet
	call	ComWrite			; won't help, so don't even try
	jmp	abort
sendSync:
	; SEND A SYNC AFTER READING BLOCK OF DATA
	mov	al, SYNC
	call	ComWrite


	; otherwise do a subtraction of a 16 bit value from a 32 bit value
	; subtract the low parts, and on carry, decrement the high part
	; of the 32 bit value
	mov	ax, cs:[filesize].low
	cmp	cx, ax
	jbe	easySubtract
	cmp 	cs:[filesize].high, 0
	jz	fileReadComplete
	dec	cs:[filesize].high
easySubtract:
	sub	ax, cx
	mov	cs:[filesize].low, ax

	or	ax, cs:[filesize].high
	jnz	readBlock

fileReadComplete:
	;NOW RECIEVE THE CHECKSUM and see if it matches what we
	; think it should be
	call	ComReadWithWait
	jc	abort
	mov	bl, al
	call	ComReadWithWait
	jc	abort
	mov	bh, al

	mov	al, ACK		; get ready to send ACK
	cmp	cs:[checksum], bx
	jz	sendAck
	mov	al, NAK
sendAck:
	call	ComWrite
	call	ComDrainQueue

done::
	mov	bx, cs:[fileHandle]
	jz	Exit
	mov	ah, DOS_FILE_CLOSE
	int	21h		; DOS FILE CLOSE
Exit:
	.leave
	ret
abort:
	call	Abort
	jmp	Exit
fileCreateError:
	call	FileCreateError
	jmp	Exit
GetFile	endp

program	ends
